* ptst.asm
* Hardware tests for the GP20 Pod
* Copyright 1999, 2001 Jed Margolin
* All Rights Reserved
* Enable the desired test, then reassemble and download to the pod
* tst6 is originally enabled (Power-Up the LCD and put out some characters)

#pagelength 65t
#pagewidth 124t
$base 10t

$INCLUDE 'podequ.h'    	; Equates for MC68HC908GP20 MCU
$INCLUDE 'pod.h'  	; Equates for GP20 Pod Jump Table
*******************************************************
* Program area starts here
*******************************************************
        ORG    USER_EMRAM 	   ;Start of Emulation RAM

data	RMB	1
*address	RMB     2		; address High, Address Low
* Start of User Program
*====================================================
* First initialize any control registers and variables
START:  RSP
* ====================================================
$SETNOT TST1
#IF TST1
* Toggle SWEN
	JSR	WAIT1ms

	LDA	#$FF
	STA	DDRC

mlp:	BSET	SWEN,PORTC
	JSR	WAIT100us

	BCLR	SWEN,PORTC
	JSR	WAIT100us

	BRA	mlp
#ENDIF
* ====================================================
$SETNOT TST2
#IF TST2
* Walking Ones through LATCH, d0 is wider 
tst2:	LDA	#$10		; make sure LATCH is high and LCDEN is low
	STA	PORTD
	LDA	#$30
	STA	DDRD

	LDA	#$FF		; Port A Outputs
	STA	DDRA

t2br1:	LDA	#$01
	STA	PORTA
	BCLR	4,PORTD
	BSET	4,PORTD

	MUL

	LDA	#$01
	LDX	#$08
t2br2:	LSLA

	STA	PORTA
	BCLR	4,PORTD
	BSET	4,PORTD

	DECX
	BNE	t2br2
	BRA	t2br1
#ENDIF
* ====================================================
$SETNOT TST3
#IF TST3
* Walking Ones through Port B, d0 is wider 
tst3:	LDA	#$10		; make sure LATCH is high and LCDEN is low
	STA	PORTD
	LDA	#$30
	STA	DDRD

	LDA	#$FF		; Port B Outputs
	STA	DDRB

t3br1:	LDA	#$01
	STA	PORTB
	BSET	5,PORTD
	BCLR	5,PORTD

	MUL

	LDA	#$01
	LDX	#$08
t3br2:	LSLA

	STA	PORTB
	BSET	5,PORTD
	BCLR	5,PORTD

	DECX
	BNE	t3br2
	BRA	t3br1
#ENDIF
* ====================================================
$SETNOT TST4
#IF TST4
* Power-Up the LCD
tst4:	
	LDA	#$10		; make sure LATCH is high and LCDEN is low
	STA	PORTD
	LDA	#$30
	STA	DDRD

	LDA	#$FF		; Port B Outputs
	STA	DDRB

	JSR	pwr_lcd2

t4lp:	BRA	t4lp
*--------------------------------
pwr_lcd2:
        JSR     WAIT10ms        ; wait 10 ms    <-----
        JSR     WAIT10ms        ; wait 10 ms    <-----
*--
        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd2_pwr_instr   ; write instruction (power-up)
        JSR     WAIT10ms        ; wait 10 ms    <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd2_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd2_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        LDA     #$38            ; 8bit interface, 2 lines, 5x7 dots
        BSR     lcd2_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$08            ; display off
        BSR     lcd2_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$01            ; display on
        BSR     lcd2_pwr_instr   ; write instruction (power-up)
        JSR     WAIT10ms        ; wait 10 ms    <-----
*--
*       LDA     #$0E            ; display on, cursor on, blink off
        LDA     #$0F            ; display on, cursor on, blink on
*       LDA     #$0C            ; display on, cursor off, blink off
        BSR     lcd2_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----
*--
        LDA     #$06            ; entry mode set: cursor increment, no shift
        BSR     lcd2_pwr_instr   ; write instruction (power-up)
        JSR     WAIT200us       ; <-----

        RTS
*--------------------------------------------------------------------------*
* Write instruction byte from accumulator during lcd2 powerup
lcd2_pwr_instr:
	LDX	#$FF
	STX	PORTC	; make sure IDE Interface is deselected

	LDX	#$FF
	STX	DDRA	; Command - Port A Outputs
	STX	DDRB	; Data    - Port B Outputs

	BCLR	LCDEN,PORTD	; EN = 0
	LDX	#$0F
	STX	PORTA		; RS, R/W = 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD

	STA	PORTB		; instruction byte

        BSET    LCDEN,PORTD     ; E
        NOP                     ; Wait 1 us <-----
        BCLR    LCDEN,PORTD     ; E

	CLR	DDRA		; Port A Inputs (default)
	CLR	DDRB		; Port B Inputs (default)

        JSR     WAIT100us       ; <-----
        RTS
*--------------------------------------------------------------------------*
* Write instruction byte from accumulator, use Wait for Busy
lcd2_wr_instr:
	STA	data
	LDA	#$FF
	STA	DDRA		; Command - Port A Outputs
	STA	DDRB		; Data    - Port B Outputs

	BCLR	LCDEN,PORTD	; EN = 0
	LDA	#$0F
	STA	PORTA		; RS, R/W = 0
	BCLR	LATCH,PORTD
	BSET	LATCH,PORTD

	LDA	data
	STA	PORTB		; instruction byte

        BSET    LCDEN,PORTD     ; E
        NOP                     ; Wait 1 us <-----
        BCLR    LCDEN,PORTD     ; E

	CLR	DDRA		; Port A Inputs (default)
	CLR	DDRB		; Port B Inputs (default)

        BSR     WAIT_BUSY        ; <-----

        RTS
*--------------------------------------------------------------------------*
* just do a timed wait
WAIT_BUSY:
	JSR	WAIT100us
	RTS
*--------------------------------------------------------------------------*
#ENDIF
* ====================================================
$SETNOT TST5
#IF TST5
* Power-Up the LCD
tst5:	
	LDA	#$10		; make sure LATCH is high and LCDEN is low
	STA	PORTD
	LDA	#$30
	STA	DDRD

	LDA	#$FF		; Port B Outputs
	STA	DDRB

	JSR	pwr_lcd

t5lp:	BRA	t5lp
#ENDIF
*====================================================
$SET TST6
#IF TST6
* Power-Up the LCD and put out some characters
tst6:	
	LDA	#$10		; make sure LATCH is high and LCDEN is low
	STA	PORTD
	LDA	#$30
	STA	DDRD

	LDA	#$FF		; Port B Outputs
	STA	DDRB

	JSR	pwr_lcd

* Display a message using HC08 DOMSG (with H:X)
* X = #MSG
	LDHX	#MSG0
        JSR     DOMSG

*	JMP	CMD
t6lp:	BRA	t6lp

MSG0:  FCB  $80,'I am alive.   Alive!  ',$FF

#ENDIF
*====================================================
* Do the TOC Test
$SETNOT TST7
#IF TST7
tst7:	
	LDA	#$10		; make sure LATCH is high and LCDEN is low
	STA	PORTD
	LDA	#$30
	STA	DDRD

	LDA	#$FF		; Port B Outputs
	STA	DDRB

*	JSR	pwr_lcd

	JSR	TEST_TOC
*	JMP	CMD
t7lp:	BRA	t7lp
#ENDIF
*====================================================
* Do the Play MSF Test
$SETNOT TST8
#IF TST8
tst8:	
	LDA	#$10		; make sure LATCH is high and LCDEN is low
	STA	PORTD
	LDA	#$30
	STA	DDRD

	LDA	#$FF		; Port B Outputs
	STA	DDRB

*	JSR	pwr_lcd

	JSR	TEST_PLAYMSF

*	JMP	CMD
t8lp:	BRA	t8lp
#ENDIF
*====================================================
* Do the Switch 1 Test   (Display Switch State)
$SETNOT TST9
#IF TST9
tst9:	
	JSR	TEST_SW_A

*	JMP	CMD
t9lp:	BRA	t9lp
#ENDIF
*====================================================
* Do the Switch 2 Test  (Display Positive Transitions)
$SETNOT TST10
#IF TST10
tst10:	
	JSR	TEST_SW_B

*	JMP	CMD
t10lp:	BRA	t10lp
#ENDIF
*====================================================
*====================================================
* Test JMP  ,X 		(it works)
$SETNOT TST11
#IF TST11
tst11:	
	LDA	#$10		; make sure LATCH is high and LCDEN is low
	STA	PORTD
	LDA	#$30
	STA	DDRD

	LDA	#$FF		; Port B Outputs
	STA	DDRB

	JSR	pwr_lcd

	LDHX	#ttbr1
	JMP	,X

	JSR	t11m0
	BRA	spin

ttbr1:	JSR	t11m0		; ok

spin:	BRA	spin
*--------------------------------
t11m0:
* Display a message using HC08 DOMSG (with H:X)
* X = #MSG
	LDHX	#MSG0
        JSR     DOMSG
	RTS
*--------------------------------
MSG0:  FCB  $80,'We Got Here     ',$FF

#ENDIF
*====================================================
UNUSED  RTI
