* switch3.asm
* Develop Software for the GP20 Pod
* Copyright 1999 Jed Margolin
* All Rights Reserved
#pagelength 65t
#pagewidth 124t
$base 10t

$INCLUDE 'podequ.h'    	; Equates for MC68HC908GP20 MCU
$INCLUDE 'pod.h'  	; Equates for GP20 Pod Jump Table

$SET FLAG		; display type
*==========================================================================
*******************************************************
* Program area starts here
*******************************************************

        ORG    USER_EMRAM 	   ;Start of Emulation RAM

* Start of User Program
*====================================================
* Read the switches, display positive transitions.
* First initialize any control registers and variables
START:  RSP

	LDA	#$80
        JSR     CLR_DISP

	LDA	#$C0
        JSR     CLR_DISP

	JSR	INIT_SWITCHES2

	CLR	pcount

mlp:	JSR	DO_SWITCHES2

$IF FLAG
	JSR	Display_Switch2		; positive switch transition
$ENDIF

$IFNOT FLAG
	JSR	display
$ENDIF

        JSR     WAIT10ms

	BSR	SWTRANS2

	BRA	mlp
* ====================================================
* Read and debounce the switches
DO_SWITCHES2:

	LDA	#$FF
	STA	PORTC		; IDE Interface deselected
	CLR	DDRA		; Port A Inputs

	BCLR	SWEN,PORTC	; Switches selected
        LDA     PORTA
	BSET	SWEN,PORTC	; Switches deselected
	STA	swtemp		; switch: 0 = on

* sw1 =debounce register
* sw1+1: d2=positive transition, d1=old state, d0=debounced state, 
*---------------
	LDHX	#sw1		
	BSR	swsub		; switch 1	
*---------------
	LDHX	#sw2		
	BSR	swsub		; switch 2
*---------------
	LDHX	#sw3		
	BSR	swsub		; switch 3
*---------------
	LDHX	#sw4		
	BSR	swsub		; switch 4
*---------------
	LDHX	#sw5		
	BSR	swsub		; switch 5
*---------------
	LDHX	#sw6		
	BSR	swsub		; switch 6
*---------------
	RTS
*----------------------------------
* debounce switch, check for positive transition 
swsub:
* get the two switch variables (it's faster this way)
	CLRH			; clear X high register
	LDA	,X		
	STA	swtemp+1

	LDA	1,X
	STA	swtemp+2

* look at the switch
	ROR	swtemp		; switch state is in Carry
        ROL     swtemp+1	; debounce register for the particular switch
	BEQ	sws1		; all zero

	LDA	swtemp+1	; debounce register
	CMP	#$FF
	BNE	swsout		; not all zeros, not all ones: do nothing

	BSET	0,swtemp+2	; all ones: set swtemp+2(bit 0)
	BRA	sws2

sws1:	BCLR	0,swtemp+2	; all zeros: clear bit 0, swtemp+2

* switch is debounced, state is in swtemp+2(d0)
sws2:	BRSET	0,swtemp+2,sws3	; switch not pressed, update old switch state

* Switch is pressed, is it a positive transition?

* If the old switch was pressed, it is not a transistion
	BRCLR	1,swtemp+2,swsout	

* We have a winner, set the positive transition flag
* (Whoever uses the flag is responsible for clearing it)
	BSET	2,swtemp+2

* put new state into old state
sws3:   BRSET	0,swtemp+2,sws4
	BCLR	1,swtemp+2		
	BRA	swsout

sws4:	BSET	1,swtemp+2

* Write switch parameters back to the current switch
swsout:	LDA	swtemp+1		
	STA	,X

	LDA	swtemp+2
	STA	1,X

	RTS
* ===================================================
* for testing transitions
SWTRANS2:
	TST	pcount
	BEQ	ts1

	DEC	pcount
	BNE	tsout2

* clear all the transition flags
	BCLR	2,sw1+1
	BCLR	2,sw2+1
	BCLR	2,sw3+1
	BCLR	2,sw4+1
	BCLR	2,sw5+1
	BCLR	2,sw6+1
	RTS

* look for a transition
ts1:	BRSET	2,sw1+1,tsout1
	BRSET	2,sw2+1,tsout1
	BRSET	2,sw3+1,tsout1
	BRSET	2,sw4+1,tsout1
	BRSET	2,sw5+1,tsout1
	BRSET	2,sw6+1,tsout1
	RTS

* we had a transition, start the counter
tsout1:	LDA	#40
	STA	pcount
tsout2:	RTS
* ====================================================
$IFNOT FLAG
display:
	LDA	sw1+1
        LDX     #$80    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	sw2+1
        LDX     #$83    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	sw3+1
        LDX     #$86    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	sw4+1
        LDX     #$89    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	sw5+1
        LDX     #$8C    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	sw6+1
        LDX     #$8F    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	LDA	pcount
        LDX     #$C0    	; set cursor location
        JSR     DISPLAY_BINARY  ; Display Binary from ACC

	RTS
$ENDIF
* ====================================================
$IF FLAG
* positive switch transition
Display_Switch2		
	LDA	#$80
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw1+1,tswbr1
	LDA	#'0'
tswbr1: JSR     lcd_wr_data     ; put out the character

	LDA	#$83
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw2+1,tswbr2
	LDA	#'0'
tswbr2: JSR     lcd_wr_data     ; put out the character

	LDA	#$86
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw3+1,tswbr3
	LDA	#'0'
tswbr3: JSR     lcd_wr_data     ; put out the character

	LDA	#$89
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw4+1,tswbr4
	LDA	#'0'
tswbr4: JSR     lcd_wr_data     ; put out the character

	LDA	#$8C
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw5+1,tswbr5
	LDA	#'0'
tswbr5: JSR     lcd_wr_data     ; put out the character

	LDA	#$8F
        JSR     lcd_wr_instr	; address

	LDA	#'1'
	BRSET	2,sw6+1,tswbr6
	LDA	#'0'
tswbr6: JSR     lcd_wr_data     ; put out the character

	RTS
$ENDIF
*===========================================================================
INIT_SWITCHES2:
	LDA	#$FF
	STA	sw1
	STA	sw2
	STA	sw3
	STA	sw4
	STA	sw5
	STA	sw6
	
	LDA	#$03
	STA	sw1+1
	STA	sw2+1
	STA	sw3+1
	STA	sw4+1
	STA	sw5+1
	STA	sw6+1

	RTS
*===========================================================================
UNUSED  RTI
* ====================================================
